@@ -42,10 +42,10 @@ import (
 	"cmd/go/internal/imports"
 	"cmd/go/internal/modfetch"
 	"cmd/go/internal/modload"
-	"cmd/go/internal/par"
 	"cmd/go/internal/search"
 	"cmd/go/internal/toolchain"
 	"cmd/go/internal/work"
+	"cmd/internal/par"
 
 	"golang.org/x/mod/modfile"
 	"golang.org/x/mod/module"
@@ -54,7 +54,7 @@ import (
 var CmdGet = &base.Command{
 	// Note: flags below are listed explicitly because they're the most common.
 	// Do not send CLs removing them because they're covered by [get flags].
-	UsageLine: "go get [-t] [-u] [-v] [build flags] [packages]",
+	UsageLine: "go get [-t] [-u] [-tool] [build flags] [packages]",
 	Short:     "add dependencies to current module and install them",
 	Long: `
 Get resolves its command-line arguments to packages at specific module versions,
@@ -109,6 +109,9 @@ but changes the default to select patch releases.
 When the -t and -u flags are used together, get will update
 test dependencies as well.
 
+The -tool flag instructs go to add a matching tool line to go.mod for each
+listed package. If -tool is used with @none, the line will be removed.
+
 The -x flag prints commands as they are executed. This is useful for
 debugging version control commands when a module is downloaded directly
 from a repository.
@@ -122,11 +125,6 @@ suggested Go toolchain, see https://go.dev/doc/toolchain.
 
 For more about specifying packages, see 'go help packages'.
 
-This text describes the behavior of get using modules to manage source
-code and dependencies. If instead the go command is running in GOPATH
-mode, the details of get's flags and effects change, as does 'go help get'.
-See 'go help gopath-get'.
-
 See also: go build, go install, go clean, go mod.
 	`,
 }
@@ -217,8 +215,8 @@ var (
 	getM        = CmdGet.Flag.Bool("m", false, "")
 	getT        = CmdGet.Flag.Bool("t", false, "")
 	getU        upgradeFlag
+	getTool     = CmdGet.Flag.Bool("tool", false, "")
 	getInsecure = CmdGet.Flag.Bool("insecure", false, "")
-	// -v is cfg.BuildV
 )
 
 // upgradeFlag is a custom flag.Value for -u.
@@ -338,6 +336,8 @@ func runGet(ctx context.Context, cmd *base.Command, args []string) {
 	r := newResolver(ctx, queries)
 	r.performLocalQueries(ctx)
 	r.performPathQueries(ctx)
+	r.performToolQueries(ctx)
+	r.performWorkQueries(ctx)
 
 	for {
 		r.performWildcardQueries(ctx)
@@ -400,8 +400,15 @@ func runGet(ctx context.Context, cmd *base.Command, args []string) {
 			pkgPatterns = append(pkgPatterns, q.pattern)
 		}
 	}
+
+	// If a workspace applies, checkPackageProblems will switch to the workspace
+	// using modload.EnterWorkspace when doing the final load, and then switch back.
 	r.checkPackageProblems(ctx, pkgPatterns)
 
+	if *getTool {
+		updateTools(ctx, queries, &opts)
+	}
+
 	// Everything succeeded. Update go.mod.
 	oldReqs := reqsFromGoMod(modload.ModFile())
 
@@ -425,6 +432,32 @@ func runGet(ctx context.Context, cmd *base.Command, args []string) {
 	}
 }
 
+func updateTools(ctx context.Context, queries []*query, opts *modload.WriteOpts) {
+	pkgOpts := modload.PackageOpts{
+		VendorModulesInGOROOTSrc: true,
+		LoadTests:                *getT,
+		ResolveMissingImports:    false,
+		AllowErrors:              true,
+		SilenceNoGoErrors:        true,
+	}
+	patterns := []string{}
+	for _, q := range queries {
+		if search.IsMetaPackage(q.pattern) || q.pattern == "toolchain" {
+			base.Fatalf("go: go get -tool does not work with \"%s\".", q.pattern)
+		}
+		patterns = append(patterns, q.pattern)
+	}
+
+	matches, _ := modload.LoadPackages(ctx, pkgOpts, patterns...)
+	for i, m := range matches {
+		if queries[i].version == "none" {
+			opts.DropTools = append(opts.DropTools, m.Pkgs...)
+		} else {
+			opts.AddTools = append(opts.DropTools, m.Pkgs...)
+		}
+	}
+}
+
 // parseArgs parses command-line arguments and reports errors.
 //
 // The command-line arguments are of the form path@version or simply path, with
@@ -481,6 +514,8 @@ type resolver struct {
 	pathQueries       []*query // package path literal queries in original order
 	wildcardQueries   []*query // path wildcard queries in original order
 	patternAllQueries []*query // queries with the pattern "all"
+	workQueries       []*query // queries with the pattern "work"
+	toolQueries       []*query // queries with the pattern "tool"
 
 	// Indexed "none" queries. These are also included in the slices above;
 	// they are indexed here to speed up noneForPath.
@@ -502,6 +537,10 @@ type resolver struct {
 	work *par.Queue
 
 	matchInModuleCache par.ErrCache[matchInModuleKey, []string]
+
+	// workspace is used to check whether, in workspace mode, any of the workspace
+	// modules would contain a package.
+	workspace *workspace
 }
 
 type versionReason struct {
@@ -535,11 +574,16 @@ func newResolver(ctx context.Context, queries []*query) *resolver {
 		buildListVersion: initialVersion,
 		initialVersion:   initialVersion,
 		nonesByPath:      map[string]*query{},
+		workspace:        loadWorkspace(modload.FindGoWork(base.Cwd())),
 	}
 
 	for _, q := range queries {
 		if q.pattern == "all" {
 			r.patternAllQueries = append(r.patternAllQueries, q)
+		} else if q.pattern == "work" {
+			r.workQueries = append(r.workQueries, q)
+		} else if q.pattern == "tool" {
+			r.toolQueries = append(r.toolQueries, q)
 		} else if q.patternIsLocal {
 			r.localQueries = append(r.localQueries, q)
 		} else if q.isWildcard() {
@@ -725,8 +769,9 @@ func (r *resolver) performLocalQueries(ctx context.Context) {
 			pkgPattern, mainModule := modload.MainModules.DirImportPath(ctx, q.pattern)
 			if pkgPattern == "." {
 				modload.MustHaveModRoot()
-				var modRoots []string
-				for _, m := range modload.MainModules.Versions() {
+				versions := modload.MainModules.Versions()
+				modRoots := make([]string, 0, len(versions))
+				for _, m := range versions {
 					modRoots = append(modRoots, modload.MainModules.ModRoot(m))
 				}
 				var plural string
@@ -1016,6 +1061,50 @@ func (r *resolver) queryPath(ctx context.Context, q *query) {
 	})
 }
 
+// performToolQueries populates the candidates for each query whose
+// pattern is "tool".
+func (r *resolver) performToolQueries(ctx context.Context) {
+	for _, q := range r.toolQueries {
+		for tool := range modload.MainModules.Tools() {
+			q.pathOnce(tool, func() pathSet {
+				pkgMods, err := r.queryPackages(ctx, tool, q.version, r.initialSelected)
+				return pathSet{pkgMods: pkgMods, err: err}
+			})
+		}
+	}
+}
+
+// performWorkQueries populates the candidates for each query whose pattern is "work".
+// The candidate module to resolve the work pattern is exactly the single main module.
+func (r *resolver) performWorkQueries(ctx context.Context) {
+	for _, q := range r.workQueries {
+		q.pathOnce(q.pattern, func() pathSet {
+			// TODO(matloob): Maybe export MainModules.mustGetSingleMainModule and call that.
+			// There are a few other places outside the modload package where we expect
+			// a single main module.
+			if len(modload.MainModules.Versions()) != 1 {
+				panic("internal error: number of main modules is not exactly one in resolution phase of go get")
+			}
+			mainModule := modload.MainModules.Versions()[0]
+
+			// We know what the result is going to be, assuming the main module is not
+			// empty, (it's the main module itself) but first check to see that there
+			// are packages in the main module, so that if there aren't any, we can
+			// return the expected warning that the pattern matched no packages.
+			match := modload.MatchInModule(ctx, q.pattern, mainModule, imports.AnyTags())
+			if len(match.Errs) > 0 {
+				return pathSet{err: match.Errs[0]}
+			}
+			if len(match.Pkgs) == 0 {
+				search.WarnUnmatched([]*search.Match{match})
+				return pathSet{} // There are no packages in the main module, so the main module isn't needed to resolve them.
+			}
+
+			return pathSet{pkgMods: []module.Version{mainModule}}
+		})
+	}
+}
+
 // performPatternAllQueries populates the candidates for each query whose
 // pattern is "all".
 //
@@ -1187,12 +1276,12 @@ func (r *resolver) loadPackages(ctx context.Context, patterns []string, findPack
 	}
 
 	_, pkgs := modload.LoadPackages(ctx, opts, patterns...)
-	for _, path := range pkgs {
+	for _, pkgPath := range pkgs {
 		const (
 			parentPath  = ""
 			parentIsStd = false
 		)
-		_, _, err := modload.Lookup(parentPath, parentIsStd, path)
+		_, _, err := modload.Lookup(parentPath, parentIsStd, pkgPath)
 		if err == nil {
 			continue
 		}
@@ -1200,6 +1289,10 @@ func (r *resolver) loadPackages(ctx context.Context, patterns []string, findPack
 			// We already added candidates during loading.
 			continue
 		}
+		if r.workspace != nil && r.workspace.hasPackage(pkgPath) {
+			// Don't try to resolve imports that are in the resolver's associated workspace. (#73654)
+			continue
+		}
 
 		var (
 			importMissing *modload.ImportMissingError
@@ -1212,7 +1305,7 @@ func (r *resolver) loadPackages(ctx context.Context, patterns []string, findPack
 			continue
 		}
 
-		path := path
+		path := pkgPath
 		r.work.Add(func() {
 			findPackage(ctx, path, module.Version{})
 		})
@@ -1511,6 +1604,27 @@ func (r *resolver) chooseArbitrarily(cs pathSet) (isPackage bool, m module.Versi
 func (r *resolver) checkPackageProblems(ctx context.Context, pkgPatterns []string) {
 	defer base.ExitIfErrors()
 
+	// Enter workspace mode, if the current main module would belong to it, when
+	// doing the workspace load. We want to check that the workspace loads properly
+	// and doesn't have missing or ambiguous imports (rather than checking the module
+	// by itself) because the module may have unreleased dependencies in the workspace.
+	// We'll also report issues for retracted and deprecated modules using the workspace
+	// info, but switch back to single module mode when fetching sums so that we update
+	// the single module's go.sum file.
+	var exitWorkspace func()
+	if r.workspace != nil && r.workspace.hasModule(modload.MainModules.Versions()[0].Path) {
+		var err error
+		exitWorkspace, err = modload.EnterWorkspace(ctx)
+		if err != nil {
+			// A TooNewError can happen for
+			// go get go@newversion when all the required modules
+			// are old enough but the go command itself is not new
+			// enough. See the related comment on the SwitchOrFatal
+			// in runGet when WriteGoMod returns an error.
+			toolchain.SwitchOrFatal(ctx, err)
+		}
+	}
+
 	// Gather information about modules we might want to load retractions and
 	// deprecations for. Loading this metadata requires at least one version
 	// lookup per module, and we don't want to load information that's neither
@@ -1520,7 +1634,7 @@ func (r *resolver) checkPackageProblems(ctx context.Context, pkgPatterns []strin
 		resolved modFlags = 1 << iota // version resolved by 'go get'
 		named                         // explicitly named on command line or provides a named package
 		hasPkg                        // needed to build named packages
-		direct                        // provides a direct dependency of the main module
+		direct                        // provides a direct dependency of the main module or workspace modules
 	)
 	relevantMods := make(map[module.Version]modFlags)
 	for path, reason := range r.resolvedVersion {
@@ -1611,8 +1725,8 @@ func (r *resolver) checkPackageProblems(ctx context.Context, pkgPatterns []strin
 
 	// Load deprecations for modules mentioned on the command line. Only load
 	// deprecations for indirect dependencies if they're also direct dependencies
-	// of the main module. Deprecations of purely indirect dependencies are
-	// not actionable.
+	// of the main module or workspace modules. Deprecations of purely indirect
+	// dependencies are not actionable.
 	deprecations := make([]modMessage, 0, len(relevantMods))
 	for m, flags := range relevantMods {
 		if flags&(resolved|named) != 0 || flags&(hasPkg|direct) == hasPkg|direct {
@@ -1631,6 +1745,16 @@ func (r *resolver) checkPackageProblems(ctx context.Context, pkgPatterns []strin
 		})
 	}
 
+	// exit the workspace if we had entered it earlier. We want to add the sums
+	// to the go.sum file for the module we're running go get from.
+	if exitWorkspace != nil {
+		// Wait for retraction and deprecation checks (that depend on the global
+		// modload state containing the workspace) to finish before we reset the
+		// state back to single module mode.
+		<-r.work.Idle()
+		exitWorkspace()
+	}
+
 	// Load sums for updated modules that had sums before. When we update a
 	// module, we may update another module in the build list that provides a
 	// package in 'all' that wasn't loaded as part of this 'go get' command.
@@ -1954,3 +2078,53 @@ func isNoSuchPackageVersion(err error) bool {
 	var noPackage *modload.PackageNotInModuleError
 	return isNoSuchModuleVersion(err) || errors.As(err, &noPackage)
 }
+
+// workspace represents the set of modules in a workspace.
+// It can be used
+type workspace struct {
+	modules map[string]string // path -> modroot
+}
+
+// loadWorkspace loads infomation about a workspace using a go.work
+// file path.
+func loadWorkspace(workFilePath string) *workspace {
+	if workFilePath == "" {
+		// Return the empty workspace checker. All HasPackage checks will return false.
+		return nil
+	}
+
+	_, modRoots, err := modload.LoadWorkFile(workFilePath)
+	if err != nil {
+		return nil
+	}
+
+	w := &workspace{modules: make(map[string]string)}
+	for _, modRoot := range modRoots {
+		modFile := filepath.Join(modRoot, "go.mod")
+		_, f, err := modload.ReadModFile(modFile, nil)
+		if err != nil {
+			continue // Error will be reported in the final load of the workspace.
+		}
+		w.modules[f.Module.Mod.Path] = modRoot
+	}
+
+	return w
+}
+
+// hasPackage reports whether there is a workspace module that could
+// provide the package with the given path.
+func (w *workspace) hasPackage(pkgpath string) bool {
+	for modPath, modroot := range w.modules {
+		if modload.PkgIsInLocalModule(pkgpath, modPath, modroot) {
+			return true
+		}
+	}
+	return false
+}
+
+// hasModule reports whether there is a workspace module with the given
+// path.
+func (w *workspace) hasModule(modPath string) bool {
+	_, ok := w.modules[modPath]
+	return ok
+}
