@@ -8,7 +8,7 @@ import (
 	"cmd/go/internal/cfg"
 	"cmd/go/internal/gover"
 	"cmd/go/internal/mvs"
-	"cmd/go/internal/par"
+	"cmd/internal/par"
 	"context"
 	"errors"
 	"fmt"
@@ -681,7 +681,7 @@ type dqTracker struct {
 	// in the extended module graph.
 	extendedRootPruning map[module.Version]modPruning
 
-	// dqReason records whether and why each each encountered version is
+	// dqReason records whether and why each encountered version is
 	// disqualified in a pruned or unpruned context.
 	dqReason map[module.Version]perPruning[dqState]
 
@@ -749,7 +749,7 @@ func (t *dqTracker) require(m, r module.Version) (ok bool) {
 		}
 	}
 
-	// Record that m is a dependant of r, so that if r is later disqualified
+	// Record that m is a dependent of r, so that if r is later disqualified
 	// m will be disqualified as well.
 	if t.requiring == nil {
 		t.requiring = make(map[module.Version][]module.Version)
