@@ -60,18 +60,27 @@ func TestRuntimeTypesPresent(t *testing.T) {
 		"internal/abi.ArrayType":     true,
 		"internal/abi.ChanType":      true,
 		"internal/abi.FuncType":      true,
-		"internal/abi.MapType":       true,
 		"internal/abi.PtrType":       true,
 		"internal/abi.SliceType":     true,
 		"internal/abi.StructType":    true,
 		"internal/abi.InterfaceType": true,
-		"runtime.itab":               true,
+		"internal/abi.ITab":          true,
 	}
 
 	found := findTypes(t, dwarf, want)
 	if len(found) != len(want) {
 		t.Errorf("found %v, want %v", found, want)
 	}
+
+	// Must have one of OldMapType or SwissMapType.
+	want = map[string]bool{
+		"internal/abi.OldMapType":   true,
+		"internal/abi.SwissMapType": true,
+	}
+	found = findTypes(t, dwarf, want)
+	if len(found) != 1 {
+		t.Errorf("map type want one of %v found %v", want, found)
+	}
 }
 
 func findTypes(t *testing.T, dw *dwarf.Data, want map[string]bool) (found map[string]bool) {
@@ -122,8 +131,8 @@ func gobuild(t *testing.T, dir string, testfile string, gcflags string) *builtFi
 
 // Similar to gobuild() above, but uses a main package instead of a test.go file.
 
-func gobuildTestdata(t *testing.T, tdir string, pkgDir string, gcflags string) *builtFile {
-	dst := filepath.Join(tdir, "out.exe")
+func gobuildTestdata(t *testing.T, pkgDir string, gcflags string) *builtFile {
+	dst := filepath.Join(t.TempDir(), "out.exe")
 
 	// Run a build with an updated GOPATH
 	cmd := testenv.Command(t, testenv.GoToolPath(t), "build", gcflags, "-o", dst)
@@ -278,7 +287,10 @@ func TestSizes(t *testing.T) {
 	mustHaveDWARF(t)
 
 	// External linking may bring in C symbols with unknown size. Skip.
-	testenv.MustInternalLink(t, false)
+	//
+	// N.B. go build below explictly doesn't pass through
+	// -asan/-msan/-race, so we don't care about those.
+	testenv.MustInternalLink(t, testenv.NoSpecialBuildTypes)
 
 	t.Parallel()
 
@@ -753,10 +765,8 @@ func main() {
 func abstractOriginSanity(t *testing.T, pkgDir string, flags string) {
 	t.Parallel()
 
-	dir := t.TempDir()
-
 	// Build with inlining, to exercise DWARF inlining support.
-	f := gobuildTestdata(t, dir, filepath.Join(pkgDir, "main"), flags)
+	f := gobuildTestdata(t, filepath.Join(pkgDir, "main"), flags)
 	defer f.Close()
 
 	d, err := f.DWARF()
@@ -831,13 +841,7 @@ func TestAbstractOriginSanity(t *testing.T) {
 	}
 
 	mustHaveDWARF(t)
-
-	if wd, err := os.Getwd(); err == nil {
-		gopathdir := filepath.Join(wd, "testdata", "httptest")
-		abstractOriginSanity(t, gopathdir, OptAllInl4)
-	} else {
-		t.Fatalf("os.Getwd() failed %v", err)
-	}
+	abstractOriginSanity(t, "testdata/httptest", OptAllInl4)
 }
 
 func TestAbstractOriginSanityIssue25459(t *testing.T) {
@@ -848,29 +852,21 @@ func TestAbstractOriginSanityIssue25459(t *testing.T) {
 		t.Skip("skipping on not-amd64 not-386; location lists not supported")
 	}
 
-	if wd, err := os.Getwd(); err == nil {
-		gopathdir := filepath.Join(wd, "testdata", "issue25459")
-		abstractOriginSanity(t, gopathdir, DefaultOpt)
-	} else {
-		t.Fatalf("os.Getwd() failed %v", err)
-	}
+	abstractOriginSanity(t, "testdata/issue25459", DefaultOpt)
 }
 
 func TestAbstractOriginSanityIssue26237(t *testing.T) {
 	testenv.MustHaveGoBuild(t)
 
 	mustHaveDWARF(t)
-	if wd, err := os.Getwd(); err == nil {
-		gopathdir := filepath.Join(wd, "testdata", "issue26237")
-		abstractOriginSanity(t, gopathdir, DefaultOpt)
-	} else {
-		t.Fatalf("os.Getwd() failed %v", err)
-	}
+	abstractOriginSanity(t, "testdata/issue26237", DefaultOpt)
 }
 
 func TestRuntimeTypeAttrInternal(t *testing.T) {
 	testenv.MustHaveGoBuild(t)
-	testenv.MustInternalLink(t, false)
+	// N.B. go build below explictly doesn't pass through
+	// -asan/-msan/-race, so we don't care about those.
+	testenv.MustInternalLink(t, testenv.NoSpecialBuildTypes)
 
 	mustHaveDWARF(t)
 
@@ -1222,14 +1218,7 @@ func TestMachoIssue32233(t *testing.T) {
 		t.Skip("skipping; test only interesting on darwin")
 	}
 
-	tmpdir := t.TempDir()
-
-	wd, err := os.Getwd()
-	if err != nil {
-		t.Fatalf("where am I? %v", err)
-	}
-	pdir := filepath.Join(wd, "testdata", "issue32233", "main")
-	f := gobuildTestdata(t, tmpdir, pdir, DefaultOpt)
+	f := gobuildTestdata(t, "testdata/issue32233/main", DefaultOpt)
 	f.Close()
 }
 
@@ -1304,13 +1293,7 @@ func TestIssue38192(t *testing.T) {
 
 	// Build a test program that contains a translation unit whose
 	// text (from am assembly source) contains only a single instruction.
-	tmpdir := t.TempDir()
-	wd, err := os.Getwd()
-	if err != nil {
-		t.Fatalf("where am I? %v", err)
-	}
-	pdir := filepath.Join(wd, "testdata", "issue38192")
-	f := gobuildTestdata(t, tmpdir, pdir, DefaultOpt)
+	f := gobuildTestdata(t, "testdata/issue38192", DefaultOpt)
 	defer f.Close()
 
 	// Open the resulting binary and examine the DWARF it contains.
@@ -1422,14 +1405,7 @@ func TestIssue39757(t *testing.T) {
 	// compiler/runtime in ways that aren't happening now, so this
 	// might be something to check for if it does start failing.
 
-	tmpdir := t.TempDir()
-
-	wd, err := os.Getwd()
-	if err != nil {
-		t.Fatalf("where am I? %v", err)
-	}
-	pdir := filepath.Join(wd, "testdata", "issue39757")
-	f := gobuildTestdata(t, tmpdir, pdir, DefaultOpt)
+	f := gobuildTestdata(t, "testdata/issue39757", DefaultOpt)
 	defer f.Close()
 
 	syms, err := f.Symbols()
@@ -1464,9 +1440,15 @@ func TestIssue39757(t *testing.T) {
 
 	maindie := findSubprogramDIE(t, ex, "main.main")
 
-	// Collect the start/end PC for main.main
-	lowpc := maindie.Val(dwarf.AttrLowpc).(uint64)
-	highpc := maindie.Val(dwarf.AttrHighpc).(uint64)
+	// Collect the start/end PC for main.main. The format/class of the
+	// high PC attr may vary depending on which DWARF version we're generating;
+	// invoke a helper to handle the various possibilities.
+	// the low PC as opposed to an address; allow for both possibilities.
+	lowpc, highpc, perr := dwtest.SubprogLoAndHighPc(maindie)
+	if perr != nil {
+		t.Fatalf("main.main DIE malformed: %v", perr)
+	}
+	t.Logf("lo=0x%x hi=0x%x\n", lowpc, highpc)
 
 	// Now read the line table for the 'main' compilation unit.
 	mainIdx := ex.IdxFromOffset(maindie.Offset)
@@ -1514,23 +1496,17 @@ func TestIssue39757(t *testing.T) {
 
 func TestIssue42484(t *testing.T) {
 	testenv.MustHaveGoBuild(t)
-	testenv.MustInternalLink(t, false) // Avoid spurious failures from external linkers.
+	// Avoid spurious failures from external linkers.
+	//
+	// N.B. go build below explictly doesn't pass through
+	// -asan/-msan/-race, so we don't care about those.
+	testenv.MustInternalLink(t, testenv.NoSpecialBuildTypes)
 
 	mustHaveDWARF(t)
 
 	t.Parallel()
 
-	tmpdir, err := os.MkdirTemp("", "TestIssue42484")
-	if err != nil {
-		t.Fatalf("could not create directory: %v", err)
-	}
-	defer os.RemoveAll(tmpdir)
-	wd, err := os.Getwd()
-	if err != nil {
-		t.Fatalf("where am I? %v", err)
-	}
-	pdir := filepath.Join(wd, "testdata", "issue42484")
-	f := gobuildTestdata(t, tmpdir, pdir, NoOpt)
+	f := gobuildTestdata(t, "testdata/issue42484", NoOpt)
 
 	var lastAddr uint64
 	var lastFile string
@@ -2052,6 +2028,10 @@ func TestConsistentGoKindAndRuntimeType(t *testing.T) {
 		if !hasRT || rtt == 0 {
 			continue
 		}
+		// ... except unsafe.Pointer...
+		if name, _ := die.Val(intdwarf.DW_AT_name).(string); name == "unsafe.Pointer" {
+			continue
+		}
 		typesChecked++
 		// ... we want to see a meaningful DW_AT_go_kind value.
 		if val, ok := die.Val(intdwarf.DW_AT_go_kind).(int64); !ok || val == 0 {
