@@ -59,13 +59,6 @@ func Init() (*sys.Arch, ld.Arch) {
 		Gentext:          gentext,
 		Machoreloc1:      machoreloc1,
 
-		// TrampLimit is set such that we always run the trampoline
-		// generation code. This is necessary since calls to external
-		// symbols require the use of trampolines, regardless of the
-		// text size.
-		TrampLimit: 1,
-		Trampoline: trampoline,
-
 		ELF: ld.ELFArch{
 			Linuxdynld:     "/lib64/ld64.so.1",
 			LinuxdynldMusl: musl,
