@@ -272,14 +272,14 @@ func genstubs(ctxt *ld.Link, ldr *loader.Loader) {
 		for i := 0; i < relocs.Count(); i++ {
 			switch r := relocs.At(i); r.Type() {
 			case objabi.ElfRelocOffset + objabi.RelocType(elf.R_PPC64_REL24), objabi.R_CALLPOWER:
-				switch ldr.SymType(r.Sym()) {
-				case sym.SDYNIMPORT:
+				switch t := ldr.SymType(r.Sym()); {
+				case t == sym.SDYNIMPORT:
 					// This call goes through the PLT, generate and call through a PLT stub.
 					if sym, firstUse := genpltstub(ctxt, ldr, r, i, s); firstUse {
 						stubs = append(stubs, sym)
 					}
 
-				case sym.SXREF:
+				case t == sym.SXREF:
 					// Is this an ELF ABI defined function which is (in practice)
 					// generated by the linker to save/restore callee save registers?
 					// These are defined similarly for both PPC64 ELF and ELFv2.
@@ -289,7 +289,7 @@ func genstubs(ctxt *ld.Link, ldr *loader.Loader) {
 							abifuncs = append(abifuncs, sym)
 						}
 					}
-				case sym.STEXT:
+				case t.IsText():
 					targ := r.Sym()
 					if (ldr.AttrExternal(targ) && ldr.SymLocalentry(targ) != 1) || !ldr.AttrExternal(targ) {
 						// All local symbols share the same TOC pointer. This caller has a valid TOC
@@ -308,18 +308,18 @@ func genstubs(ctxt *ld.Link, ldr *loader.Loader) {
 				// GOPPC64 and -buildmode.
 				fallthrough
 			case objabi.ElfRelocOffset + objabi.RelocType(elf.R_PPC64_REL24_NOTOC):
-				switch ldr.SymType(r.Sym()) {
-				case sym.SDYNIMPORT:
+				switch rt := ldr.SymType(r.Sym()); {
+				case rt == sym.SDYNIMPORT:
 					// This call goes through the PLT, generate and call through a PLT stub.
 					if sym, firstUse := genpltstub(ctxt, ldr, r, i, s); firstUse {
 						stubs = append(stubs, sym)
 					}
 
-				case sym.SXREF:
+				case rt == sym.SXREF:
 					// TODO: This is not supported yet.
 					ldr.Errorf(s, "Unsupported NOTOC external reference call into %s", ldr.SymName(r.Sym()))
 
-				case sym.STEXT:
+				case rt.IsText():
 					targ := r.Sym()
 					if (ldr.AttrExternal(targ) && ldr.SymLocalentry(targ) <= 1) || (!ldr.AttrExternal(targ) && (!ldr.AttrShared(targ) || hasPCrel)) {
 						// This is NOTOC to NOTOC call (st_other is 0 or 1). No call stub is needed.
@@ -334,14 +334,14 @@ func genstubs(ctxt *ld.Link, ldr *loader.Loader) {
 			// Handle objects compiled with -fno-plt. Rewrite local calls to avoid indirect calling.
 			// These are 0 sized relocs. They mark the mtctr r12, or bctrl + ld r2,24(r1).
 			case objabi.ElfRelocOffset + objabi.RelocType(elf.R_PPC64_PLTSEQ):
-				if ldr.SymType(r.Sym()) == sym.STEXT {
+				if ldr.SymType(r.Sym()).IsText() {
 					// This should be an mtctr instruction. Turn it into a nop.
 					su := ldr.MakeSymbolUpdater(s)
 					const MASK_OP_MTCTR = 63<<26 | 0x3FF<<11 | 0x1FF<<1
 					rewritetonop(&ctxt.Target, ldr, su, int64(r.Off()), MASK_OP_MTCTR, OP_MTCTR)
 				}
 			case objabi.ElfRelocOffset + objabi.RelocType(elf.R_PPC64_PLTCALL):
-				if ldr.SymType(r.Sym()) == sym.STEXT {
+				if ldr.SymType(r.Sym()).IsText() {
 					// This relocation should point to a bctrl followed by a ld r2, 24(41)
 					// Convert the bctrl into a bl.
 					su := ldr.MakeSymbolUpdater(s)
@@ -474,24 +474,9 @@ func rewriteABIFuncReloc(ctxt *ld.Link, ldr *loader.Loader, tname string, r load
 	r.SetAdd(int64((n - minReg) * offMul))
 	firstUse = !ldr.AttrReachable(ts)
 	if firstUse {
-		ldr.SetAttrReachable(ts, true)
 		// This function only becomes reachable now. It has been dropped from
 		// the text section (it was unreachable until now), it needs included.
-		//
-		// Similarly, TOC regeneration should not happen for these functions,
-		// remove it from this save/restore function.
-		if ldr.AttrShared(ts) {
-			sb := ldr.MakeSymbolUpdater(ts)
-			sb.SetData(sb.Data()[8:])
-			sb.SetSize(sb.Size() - 8)
-			relocs := sb.Relocs()
-			// Only one PCREL reloc to .TOC. should be present.
-			if relocs.Count() != 1 {
-				log.Fatalf("Unexpected number of relocs in %s\n", ldr.SymName(ts))
-			}
-			sb.ResetRelocs()
-
-		}
+		ldr.SetAttrReachable(ts, true)
 	}
 	return ts, firstUse
 }
@@ -646,7 +631,7 @@ func addelfdynrel(target *ld.Target, ldr *loader.Loader, syms *ld.ArchSyms, s lo
 	case objabi.ElfRelocOffset + objabi.RelocType(elf.R_PPC64_GOT_PCREL34):
 		su := ldr.MakeSymbolUpdater(s)
 		su.SetRelocType(rIdx, objabi.R_ADDRPOWER_PCREL34)
-		if targType != sym.STEXT {
+		if !targType.IsText() {
 			ld.AddGotSym(target, ldr, syms, targ, uint32(elf.R_PPC64_GLOB_DAT))
 			su.SetRelocSym(rIdx, syms.GOT)
 			su.SetRelocAdd(rIdx, r.Add()+int64(ldr.SymGot(targ)))
@@ -772,7 +757,7 @@ func addelfdynrel(target *ld.Target, ldr *loader.Loader, syms *ld.ArchSyms, s lo
 			ld.AddGotSym(target, ldr, syms, targ, uint32(elf.R_PPC64_GLOB_DAT))
 			su.SetRelocSym(rIdx, syms.GOT)
 			su.SetRelocAdd(rIdx, r.Add()+int64(ldr.SymGot(targ)))
-		} else if targType == sym.STEXT {
+		} else if targType.IsText() {
 			if isPLT16_LO_DS {
 				// Expect an ld opcode to nop
 				rewritetonop(target, ldr, su, int64(r.Off()), MASK_OP_LD, OP_LD)
@@ -794,7 +779,7 @@ func addelfdynrel(target *ld.Target, ldr *loader.Loader, syms *ld.ArchSyms, s lo
 
 	switch r.Type() {
 	case objabi.R_ADDR:
-		if ldr.SymType(s) == sym.STEXT {
+		if ldr.SymType(s).IsText() {
 			log.Fatalf("R_ADDR relocation in text symbol %s is unsupported\n", ldr.SymName(s))
 		}
 		if target.IsPIE() && target.IsInternal() {
@@ -840,7 +825,7 @@ func addelfdynrel(target *ld.Target, ldr *loader.Loader, syms *ld.ArchSyms, s lo
 			// linking, in which case the relocation will be
 			// prepared in the 'reloc' phase and passed to the
 			// external linker in the 'asmb' phase.
-			if ldr.SymType(s) != sym.SDATA && ldr.SymType(s) != sym.SRODATA {
+			if t := ldr.SymType(s); !t.IsDATA() && !t.IsRODATA() {
 				break
 			}
 		}
@@ -1204,9 +1189,7 @@ func trampoline(ctxt *ld.Link, ldr *loader.Loader, ri int, rs, s loader.Sym) {
 	}
 	switch r.Type() {
 	case objabi.R_CALLPOWER:
-
 		// If branch offset is too far then create a trampoline.
-
 		if (ctxt.IsExternal() && ldr.SymSect(s) != ldr.SymSect(rs)) || (ctxt.IsInternal() && int64(int32(t<<6)>>6) != t) || ldr.SymValue(rs) == 0 || (*ld.FlagDebugTramp > 1 && ldr.SymPkg(s) != ldr.SymPkg(rs)) {
 			var tramp loader.Sym
 			for i := 0; ; i++ {
@@ -1244,7 +1227,7 @@ func trampoline(ctxt *ld.Link, ldr *loader.Loader, ri int, rs, s loader.Sym) {
 			}
 			if ldr.SymType(tramp) == 0 {
 				trampb := ldr.MakeSymbolUpdater(tramp)
-				ctxt.AddTramp(trampb)
+				ctxt.AddTramp(trampb, ldr.SymType(s))
 				gentramp(ctxt, ldr, trampb, rs, r.Add())
 			}
 			sb := ldr.MakeSymbolUpdater(s)
@@ -1456,7 +1439,7 @@ func archreloc(target *ld.Target, ldr *loader.Loader, syms *ld.ArchSyms, r loade
 		// local call offsets for externally generated objects are accounted for when converting into golang relocs.
 		if !hasPCrel && !ldr.AttrExternal(rs) && ldr.AttrShared(rs) && tgtName != "runtime.duffzero" && tgtName != "runtime.duffcopy" {
 			// Furthermore, only apply the offset if the target looks like the start of a function call.
-			if r.Add() == 0 && ldr.SymType(rs) == sym.STEXT {
+			if r.Add() == 0 && ldr.SymType(rs).IsText() {
 				t += 8
 			}
 		}
