@@ -23,231 +23,11 @@ import (
 func main() {
 	Package("crypto/sha1")
 	ConstraintExpr("!purego")
-	blockAMD64()
 	blockAVX2()
+	blockSHANI()
 	Generate()
 }
 
-func LOAD(index int) {
-	MOVL(Mem{Base: SI}.Offset(index*4), R10L)
-	BSWAPL(R10L)
-	MOVL(R10L, Mem{Base: SP}.Offset(index*4))
-}
-
-func SHUFFLE(index int) {
-	MOVL(Mem{Base: SP}.Offset(((index)&0xf)*4), R10L)
-	XORL(Mem{Base: SP}.Offset(((index-3)&0xf)*4), R10L)
-	XORL(Mem{Base: SP}.Offset(((index-8)&0xf)*4), R10L)
-	XORL(Mem{Base: SP}.Offset(((index-14)&0xf)*4), R10L)
-	ROLL(Imm(1), R10L)
-	MOVL(R10L, Mem{Base: SP}.Offset(((index)&0xf)*4))
-}
-
-func FUNC1(a, b, c, d, e GPPhysical) {
-	MOVL(d, R9L)
-	XORL(c, R9L)
-	ANDL(b, R9L)
-	XORL(d, R9L)
-}
-
-func FUNC2(a, b, c, d, e GPPhysical) {
-	MOVL(b, R9L)
-	XORL(c, R9L)
-	XORL(d, R9L)
-}
-
-func FUNC3(a, b, c, d, e GPPhysical) {
-	MOVL(b, R8L)
-	ORL(c, R8L)
-	ANDL(d, R8L)
-	MOVL(b, R9L)
-	ANDL(c, R9L)
-	ORL(R8L, R9L)
-}
-
-func FUNC4(a, b, c, d, e GPPhysical) {
-	FUNC2(a, b, c, d, e)
-}
-
-func MIX(a, b, c, d, e GPPhysical, konst int) {
-	ROLL(Imm(30), b)
-	ADDL(R9L, e)
-	MOVL(a, R8L)
-	ROLL(Imm(5), R8L)
-	LEAL(Mem{Base: e, Index: R10L, Scale: 1}.Offset(konst), e)
-	ADDL(R8L, e)
-}
-
-func ROUND1(a, b, c, d, e GPPhysical, index int) {
-	LOAD(index)
-	FUNC1(a, b, c, d, e)
-	MIX(a, b, c, d, e, 0x5A827999)
-}
-
-func ROUND1x(a, b, c, d, e GPPhysical, index int) {
-	SHUFFLE(index)
-	FUNC1(a, b, c, d, e)
-	MIX(a, b, c, d, e, 0x5A827999)
-}
-
-func ROUND2(a, b, c, d, e GPPhysical, index int) {
-	SHUFFLE(index)
-	FUNC2(a, b, c, d, e)
-	MIX(a, b, c, d, e, 0x6ED9EBA1)
-}
-
-func ROUND3(a, b, c, d, e GPPhysical, index int) {
-	SHUFFLE(index)
-	FUNC3(a, b, c, d, e)
-	MIX(a, b, c, d, e, 0x8F1BBCDC)
-}
-
-func ROUND4(a, b, c, d, e GPPhysical, index int) {
-	SHUFFLE(index)
-	FUNC4(a, b, c, d, e)
-	MIX(a, b, c, d, e, 0xCA62C1D6)
-}
-
-func blockAMD64() {
-	Implement("blockAMD64")
-	Attributes(NOSPLIT)
-	AllocLocal(64)
-
-	Load(Param("dig"), RBP)
-	Load(Param("p").Base(), RSI)
-	Load(Param("p").Len(), RDX)
-	SHRQ(Imm(6), RDX)
-	SHLQ(Imm(6), RDX)
-
-	LEAQ(Mem{Base: SI, Index: DX, Scale: 1}, RDI)
-	MOVL(Mem{Base: BP}.Offset(0*4), EAX)
-	MOVL(Mem{Base: BP}.Offset(1*4), EBX)
-	MOVL(Mem{Base: BP}.Offset(2*4), ECX)
-	MOVL(Mem{Base: BP}.Offset(3*4), EDX)
-	MOVL(Mem{Base: BP}.Offset(4*4), EBP)
-
-	CMPQ(RSI, RDI)
-	JEQ(LabelRef("end"))
-
-	loop_amd64()
-	end()
-}
-
-func loop_amd64() {
-	Label("loop")
-	MOVL(EAX, R11L)
-	MOVL(EBX, R12L)
-	MOVL(ECX, R13L)
-	MOVL(EDX, R14L)
-	MOVL(EBP, R15L)
-
-	ROUND1(EAX, EBX, ECX, EDX, EBP, 0)
-	ROUND1(EBP, EAX, EBX, ECX, EDX, 1)
-	ROUND1(EDX, EBP, EAX, EBX, ECX, 2)
-	ROUND1(ECX, EDX, EBP, EAX, EBX, 3)
-	ROUND1(EBX, ECX, EDX, EBP, EAX, 4)
-	ROUND1(EAX, EBX, ECX, EDX, EBP, 5)
-	ROUND1(EBP, EAX, EBX, ECX, EDX, 6)
-	ROUND1(EDX, EBP, EAX, EBX, ECX, 7)
-	ROUND1(ECX, EDX, EBP, EAX, EBX, 8)
-	ROUND1(EBX, ECX, EDX, EBP, EAX, 9)
-	ROUND1(EAX, EBX, ECX, EDX, EBP, 10)
-	ROUND1(EBP, EAX, EBX, ECX, EDX, 11)
-	ROUND1(EDX, EBP, EAX, EBX, ECX, 12)
-	ROUND1(ECX, EDX, EBP, EAX, EBX, 13)
-	ROUND1(EBX, ECX, EDX, EBP, EAX, 14)
-	ROUND1(EAX, EBX, ECX, EDX, EBP, 15)
-
-	ROUND1x(EBP, EAX, EBX, ECX, EDX, 16)
-	ROUND1x(EDX, EBP, EAX, EBX, ECX, 17)
-	ROUND1x(ECX, EDX, EBP, EAX, EBX, 18)
-	ROUND1x(EBX, ECX, EDX, EBP, EAX, 19)
-
-	ROUND2(EAX, EBX, ECX, EDX, EBP, 20)
-	ROUND2(EBP, EAX, EBX, ECX, EDX, 21)
-	ROUND2(EDX, EBP, EAX, EBX, ECX, 22)
-	ROUND2(ECX, EDX, EBP, EAX, EBX, 23)
-	ROUND2(EBX, ECX, EDX, EBP, EAX, 24)
-	ROUND2(EAX, EBX, ECX, EDX, EBP, 25)
-	ROUND2(EBP, EAX, EBX, ECX, EDX, 26)
-	ROUND2(EDX, EBP, EAX, EBX, ECX, 27)
-	ROUND2(ECX, EDX, EBP, EAX, EBX, 28)
-	ROUND2(EBX, ECX, EDX, EBP, EAX, 29)
-	ROUND2(EAX, EBX, ECX, EDX, EBP, 30)
-	ROUND2(EBP, EAX, EBX, ECX, EDX, 31)
-	ROUND2(EDX, EBP, EAX, EBX, ECX, 32)
-	ROUND2(ECX, EDX, EBP, EAX, EBX, 33)
-	ROUND2(EBX, ECX, EDX, EBP, EAX, 34)
-	ROUND2(EAX, EBX, ECX, EDX, EBP, 35)
-	ROUND2(EBP, EAX, EBX, ECX, EDX, 36)
-	ROUND2(EDX, EBP, EAX, EBX, ECX, 37)
-	ROUND2(ECX, EDX, EBP, EAX, EBX, 38)
-	ROUND2(EBX, ECX, EDX, EBP, EAX, 39)
-
-	ROUND3(EAX, EBX, ECX, EDX, EBP, 40)
-	ROUND3(EBP, EAX, EBX, ECX, EDX, 41)
-	ROUND3(EDX, EBP, EAX, EBX, ECX, 42)
-	ROUND3(ECX, EDX, EBP, EAX, EBX, 43)
-	ROUND3(EBX, ECX, EDX, EBP, EAX, 44)
-	ROUND3(EAX, EBX, ECX, EDX, EBP, 45)
-	ROUND3(EBP, EAX, EBX, ECX, EDX, 46)
-	ROUND3(EDX, EBP, EAX, EBX, ECX, 47)
-	ROUND3(ECX, EDX, EBP, EAX, EBX, 48)
-	ROUND3(EBX, ECX, EDX, EBP, EAX, 49)
-	ROUND3(EAX, EBX, ECX, EDX, EBP, 50)
-	ROUND3(EBP, EAX, EBX, ECX, EDX, 51)
-	ROUND3(EDX, EBP, EAX, EBX, ECX, 52)
-	ROUND3(ECX, EDX, EBP, EAX, EBX, 53)
-	ROUND3(EBX, ECX, EDX, EBP, EAX, 54)
-	ROUND3(EAX, EBX, ECX, EDX, EBP, 55)
-	ROUND3(EBP, EAX, EBX, ECX, EDX, 56)
-	ROUND3(EDX, EBP, EAX, EBX, ECX, 57)
-	ROUND3(ECX, EDX, EBP, EAX, EBX, 58)
-	ROUND3(EBX, ECX, EDX, EBP, EAX, 59)
-
-	ROUND4(EAX, EBX, ECX, EDX, EBP, 60)
-	ROUND4(EBP, EAX, EBX, ECX, EDX, 61)
-	ROUND4(EDX, EBP, EAX, EBX, ECX, 62)
-	ROUND4(ECX, EDX, EBP, EAX, EBX, 63)
-	ROUND4(EBX, ECX, EDX, EBP, EAX, 64)
-	ROUND4(EAX, EBX, ECX, EDX, EBP, 65)
-	ROUND4(EBP, EAX, EBX, ECX, EDX, 66)
-	ROUND4(EDX, EBP, EAX, EBX, ECX, 67)
-	ROUND4(ECX, EDX, EBP, EAX, EBX, 68)
-	ROUND4(EBX, ECX, EDX, EBP, EAX, 69)
-	ROUND4(EAX, EBX, ECX, EDX, EBP, 70)
-	ROUND4(EBP, EAX, EBX, ECX, EDX, 71)
-	ROUND4(EDX, EBP, EAX, EBX, ECX, 72)
-	ROUND4(ECX, EDX, EBP, EAX, EBX, 73)
-	ROUND4(EBX, ECX, EDX, EBP, EAX, 74)
-	ROUND4(EAX, EBX, ECX, EDX, EBP, 75)
-	ROUND4(EBP, EAX, EBX, ECX, EDX, 76)
-	ROUND4(EDX, EBP, EAX, EBX, ECX, 77)
-	ROUND4(ECX, EDX, EBP, EAX, EBX, 78)
-	ROUND4(EBX, ECX, EDX, EBP, EAX, 79)
-
-	ADDL(R11L, EAX)
-	ADDL(R12L, EBX)
-	ADDL(R13L, ECX)
-	ADDL(R14L, EDX)
-	ADDL(R15L, EBP)
-
-	ADDQ(Imm(64), RSI)
-	CMPQ(RSI, RDI)
-	JB(LabelRef("loop"))
-}
-
-func end() {
-	Label("end")
-	Load(Param("dig"), RDI)
-	MOVL(EAX, Mem{Base: DI}.Offset(0*4))
-	MOVL(EBX, Mem{Base: DI}.Offset(1*4))
-	MOVL(ECX, Mem{Base: DI}.Offset(2*4))
-	MOVL(EDX, Mem{Base: DI}.Offset(3*4))
-	MOVL(EBP, Mem{Base: DI}.Offset(4*4))
-	RET()
-}
-
 // This is the implementation using AVX2, BMI1 and BMI2. It is based on:
 // "SHA-1 implementation with Intel(R) AVX2 instruction set extensions"
 // From http://software.intel.com/en-us/articles
