@@ -2,16 +2,17 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-//go:build unix || (js && wasm) || wasip1
+//go:build unix || js || wasip1
 
 package net
 
 import "syscall"
 
 var (
-	testHookDialChannel  = func() {} // for golang.org/issue/5349
 	testHookCanceledDial = func() {} // for golang.org/issue/16523
 
+	hostsFilePath = "/etc/hosts"
+
 	// Placeholders for socket system calls.
 	socketFunc        func(int, int, int) (int, error)  = syscall.Socket
 	connectFunc       func(int, syscall.Sockaddr) error = syscall.Connect
