@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-//go:build !js && !plan9 && !wasip1
+//go:build !plan9
 
 package net
 
@@ -610,7 +610,8 @@ func TestIPv6MulticastListener(t *testing.T) {
 	if !supportsIPv6() {
 		t.Skip("IPv6 is not supported")
 	}
-	if os.Getuid() != 0 {
+	// On Windows, the test can be run by non-admin users.
+	if runtime.GOOS != "windows" && os.Getuid() != 0 {
 		t.Skip("must be root")
 	}
 
@@ -669,7 +670,7 @@ func checkMulticastListener(c *UDPConn, ip IP) error {
 
 func multicastRIBContains(ip IP) (bool, error) {
 	switch runtime.GOOS {
-	case "aix", "dragonfly", "netbsd", "openbsd", "plan9", "solaris", "illumos", "windows":
+	case "aix", "dragonfly", "netbsd", "openbsd", "plan9", "solaris", "illumos":
 		return true, nil // not implemented yet
 	case "linux":
 		if runtime.GOARCH == "arm" || runtime.GOARCH == "alpha" {
