@@ -89,38 +89,10 @@ func (fd *netFD) ctrlNetwork() string {
 	return fd.net + "6"
 }
 
-func (fd *netFD) addrFunc() func(syscall.Sockaddr) Addr {
-	switch fd.family {
-	case syscall.AF_INET, syscall.AF_INET6:
-		switch fd.sotype {
-		case syscall.SOCK_STREAM:
-			return sockaddrToTCP
-		case syscall.SOCK_DGRAM:
-			return sockaddrToUDP
-		case syscall.SOCK_RAW:
-			return sockaddrToIP
-		}
-	case syscall.AF_UNIX:
-		switch fd.sotype {
-		case syscall.SOCK_STREAM:
-			return sockaddrToUnix
-		case syscall.SOCK_DGRAM:
-			return sockaddrToUnixgram
-		case syscall.SOCK_SEQPACKET:
-			return sockaddrToUnixpacket
-		}
-	}
-	return func(syscall.Sockaddr) Addr { return nil }
-}
-
 func (fd *netFD) dial(ctx context.Context, laddr, raddr sockaddr, ctrlCtxFn func(context.Context, string, string, syscall.RawConn) error) error {
 	var c *rawConn
-	var err error
 	if ctrlCtxFn != nil {
-		c, err = newRawConn(fd)
-		if err != nil {
-			return err
-		}
+		c = newRawConn(fd)
 		var ctrlAddr string
 		if raddr != nil {
 			ctrlAddr = raddr.String()
@@ -133,6 +105,7 @@ func (fd *netFD) dial(ctx context.Context, laddr, raddr sockaddr, ctrlCtxFn func
 	}
 
 	var lsa syscall.Sockaddr
+	var err error
 	if laddr != nil {
 		if lsa, err = laddr.sockaddr(fd.family); err != nil {
 			return err
@@ -185,10 +158,7 @@ func (fd *netFD) listenStream(ctx context.Context, laddr sockaddr, backlog int,
 	}
 
 	if ctrlCtxFn != nil {
-		c, err := newRawConn(fd)
-		if err != nil {
-			return err
-		}
+		c := newRawConn(fd)
 		if err := ctrlCtxFn(ctx, fd.ctrlNetwork(), laddr.String(), c); err != nil {
 			return err
 		}
@@ -239,10 +209,7 @@ func (fd *netFD) listenDatagram(ctx context.Context, laddr sockaddr, ctrlCtxFn f
 	}
 
 	if ctrlCtxFn != nil {
-		c, err := newRawConn(fd)
-		if err != nil {
-			return err
-		}
+		c := newRawConn(fd)
 		if err := ctrlCtxFn(ctx, fd.ctrlNetwork(), laddr.String(), c); err != nil {
 			return err
 		}
