@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-//go:build openbsd && (386 || arm || amd64 || arm64 || mips64 || riscv64)
+//go:build openbsd && (386 || arm || amd64 || arm64 || riscv64)
 
 #include <sys/types.h>
 #include <pthread.h>
@@ -34,6 +34,7 @@ _cgo_sys_thread_start(ThreadStart *ts)
 	pthread_sigmask(SIG_SETMASK, &ign, &oset);
 
 	pthread_attr_init(&attr);
+	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 	pthread_attr_getstacksize(&attr, &size);
 
 	// Leave stacklo=0 and set stackhi=size; mstart will do the rest.
