@@ -5,7 +5,7 @@
 package runtime
 
 import (
-	"runtime/internal/atomic"
+	"internal/runtime/atomic"
 	"unsafe"
 )
 
@@ -18,7 +18,7 @@ const (
 // prevents us from allocating more stack.
 //
 //go:nosplit
-func sysAllocOS(n uintptr) unsafe.Pointer {
+func sysAllocOS(n uintptr, vmaName string) unsafe.Pointer {
 	p, err := mmap(nil, n, _PROT_READ|_PROT_WRITE, _MAP_ANON|_MAP_PRIVATE, -1, 0)
 	if err != 0 {
 		if err == _EACCES {
@@ -31,6 +31,7 @@ func sysAllocOS(n uintptr) unsafe.Pointer {
 		}
 		return nil
 	}
+	setVMAName(p, n, vmaName)
 	return p
 }
 
@@ -70,7 +71,10 @@ func sysUnusedOS(v unsafe.Pointer, n uintptr) {
 		// Fall back on mmap if it's not supported.
 		// _MAP_ANON|_MAP_FIXED|_MAP_PRIVATE will unmap all the
 		// pages in the old mapping, and remap the memory region.
-		mmap(v, n, _PROT_READ|_PROT_WRITE, _MAP_ANON|_MAP_FIXED|_MAP_PRIVATE, -1, 0)
+		p, err := mmap(v, n, _PROT_READ|_PROT_WRITE, _MAP_ANON|_MAP_FIXED|_MAP_PRIVATE, -1, 0)
+		if err == 0 && p != nil {
+			setVMAName(p, n, "unused")
+		}
 	}
 
 	if debug.harddecommit > 0 {
@@ -78,6 +82,7 @@ func sysUnusedOS(v unsafe.Pointer, n uintptr) {
 		if p != v || err != 0 {
 			throw("runtime: cannot disable permissions in address space")
 		}
+		setVMAName(p, n, "unused")
 	}
 }
 
@@ -90,6 +95,7 @@ func sysUsedOS(v unsafe.Pointer, n uintptr) {
 		if p != v || err != 0 {
 			throw("runtime: cannot remap pages in address space")
 		}
+		setVMAName(p, n, "used")
 		return
 	}
 }
@@ -150,18 +156,20 @@ func sysFreeOS(v unsafe.Pointer, n uintptr) {
 }
 
 func sysFaultOS(v unsafe.Pointer, n uintptr) {
-	mmap(v, n, _PROT_NONE, _MAP_ANON|_MAP_PRIVATE|_MAP_FIXED, -1, 0)
+	mprotect(v, n, _PROT_NONE)
+	madvise(v, n, _MADV_DONTNEED)
 }
 
-func sysReserveOS(v unsafe.Pointer, n uintptr) unsafe.Pointer {
+func sysReserveOS(v unsafe.Pointer, n uintptr, vmaName string) unsafe.Pointer {
 	p, err := mmap(v, n, _PROT_NONE, _MAP_ANON|_MAP_PRIVATE, -1, 0)
 	if err != 0 {
 		return nil
 	}
+	setVMAName(p, n, vmaName)
 	return p
 }
 
-func sysMapOS(v unsafe.Pointer, n uintptr) {
+func sysMapOS(v unsafe.Pointer, n uintptr, vmaName string) {
 	p, err := mmap(v, n, _PROT_READ|_PROT_WRITE, _MAP_ANON|_MAP_FIXED|_MAP_PRIVATE, -1, 0)
 	if err == _ENOMEM {
 		throw("runtime: out of memory")
@@ -170,4 +178,13 @@ func sysMapOS(v unsafe.Pointer, n uintptr) {
 		print("runtime: mmap(", v, ", ", n, ") returned ", p, ", ", err, "\n")
 		throw("runtime: cannot map pages in arena address space")
 	}
+	setVMAName(p, n, vmaName)
+
+	// Disable huge pages if the GODEBUG for it is set.
+	//
+	// Note that there are a few sysHugePage calls that can override this, but
+	// they're all for GC metadata.
+	if debug.disablethp != 0 {
+		sysNoHugePageOS(v, n)
+	}
 }
