@@ -7,14 +7,21 @@ package runtime
 import "unsafe"
 
 func sbrk(n uintptr) unsafe.Pointer {
-	grow := divRoundUp(n, physPageSize)
-	size := growMemory(int32(grow))
-	if size < 0 {
-		return nil
+	bl := bloc
+	n = memRound(n)
+	if bl+n > blocMax {
+		grow := (bl + n - blocMax) / physPageSize
+		size := growMemory(int32(grow))
+		if size < 0 {
+			return nil
+		}
+		resetMemoryDataView()
+		blocMax = bl + n
 	}
-	resetMemoryDataView()
-	return unsafe.Pointer(uintptr(size) * physPageSize)
+	bloc += n
+	return unsafe.Pointer(bl)
 }
 
 // Implemented in src/runtime/sys_wasm.s
 func growMemory(pages int32) int32
+func currentMemory() int32
