@@ -5,7 +5,6 @@
 package runtime_test
 
 import (
-	"internal/goexperiment"
 	"reflect"
 	"runtime"
 	"testing"
@@ -16,19 +15,13 @@ import (
 
 func TestSizeof(t *testing.T) {
 	const _64bit = unsafe.Sizeof(uintptr(0)) == 8
-
-	g32bit := uintptr(256)
-	if goexperiment.ExecTracer2 {
-		g32bit = uintptr(260)
-	}
-
 	var tests = []struct {
 		val    any     // type as a value
 		_32bit uintptr // size on 32bit platforms
 		_64bit uintptr // size on 64bit platforms
 	}{
-		{runtime.G{}, g32bit, 424}, // g, but exported for testing
-		{runtime.Sudog{}, 56, 88},  // sudog, but exported for testing
+		{runtime.G{}, 280, 440},   // g, but exported for testing
+		{runtime.Sudog{}, 56, 88}, // sudog, but exported for testing
 	}
 
 	for _, tt := range tests {
